import abc
import inspect
from collections.abc import AsyncGenerator, Coroutine, Generator
from typing import Callable

from acp_sdk.models import AgentName, Message, Metadata
from acp_sdk.server.context import Context
from acp_sdk.server.types import RunYield, RunYieldResume


class AgentManifest(abc.ABC):
    @property
    def name(self) -> AgentName:
        return self.__class__.__name__

    @property
    def description(self) -> str:
        return ""

    @property
    def metadata(self) -> Metadata:
        return Metadata()

    @abc.abstractmethod
    def run(
        self, input: list[Message], context: Context
    ) -> (
        AsyncGenerator[RunYield, RunYieldResume] | Generator[RunYield, RunYieldResume] | Coroutine[RunYield] | RunYield
    ):
        pass


Agent = AgentManifest


def agent(
    name: str | None = None,
    description: str | None = None,
    *,
    metadata: Metadata | None = None,
) -> Callable[[Callable], AgentManifest]:
    """Decorator to create an agent."""

    def decorator(fn: Callable) -> AgentManifest:
        signature = inspect.signature(fn)
        parameters = list(signature.parameters.values())

        if len(parameters) == 0:
            raise TypeError("The agent function must have at least 'input' argument")
        if len(parameters) > 2:
            raise TypeError("The agent function must have only 'input' and 'context' arguments")
        if len(parameters) == 2 and parameters[1].name != "context":
            raise TypeError("The second argument of the agent function must be 'context'")

        has_context_param = len(parameters) == 2

        class DecoratorAgentBase(AgentManifest):
            @property
            def name(self) -> str:
                return name or fn.__name__

            @property
            def description(self) -> str:
                return description or inspect.getdoc(fn) or ""

            @property
            def metadata(self) -> Metadata:
                return metadata or Metadata()

        agent: AgentManifest
        if inspect.isasyncgenfunction(fn):

            class AsyncGenDecoratorAgent(DecoratorAgentBase):
                async def run(self, input: list[Message], context: Context) -> AsyncGenerator[RunYield, RunYieldResume]:
                    try:
                        gen: AsyncGenerator[RunYield, RunYieldResume] = (
                            fn(input, context) if has_context_param else fn(input)
                        )
                        value = None
                        while True:
                            value = yield await gen.asend(value)
                    except StopAsyncIteration:
                        pass

            agent = AsyncGenDecoratorAgent()
        elif inspect.iscoroutinefunction(fn):

            class CoroDecoratorAgent(DecoratorAgentBase):
                async def run(self, input: list[Message], context: Context) -> Coroutine[RunYield]:
                    return await (fn(input, context) if has_context_param else fn(input))

            agent = CoroDecoratorAgent()
        elif inspect.isgeneratorfunction(fn):

            class GenDecoratorAgent(DecoratorAgentBase):
                def run(self, input: list[Message], context: Context) -> Generator[RunYield, RunYieldResume]:
                    yield from (fn(input, context) if has_context_param else fn(input))

            agent = GenDecoratorAgent()
        else:

            class FuncDecoratorAgent(DecoratorAgentBase):
                def run(self, input: list[Message], context: Context) -> RunYield:
                    return fn(input, context) if has_context_param else fn(input)

            agent = FuncDecoratorAgent()

        return agent

    return decorator



from pydantic import BaseModel

from acp_sdk.models.models import (
    AgentManifest,
    AgentName,
    AwaitResume,
    Event,
    Message,
    Run,
    RunMode,
    Session,
    SessionId,
)


class PingResponse(BaseModel):
    pass


class AgentsListResponse(BaseModel):
    agents: list[AgentManifest]


class AgentReadResponse(AgentManifest):
    pass


class RunCreateRequest(BaseModel):
    agent_name: AgentName
    session_id: SessionId | None = None
    session: Session | None = None
    input: list[Message]
    mode: RunMode = RunMode.SYNC


class RunCreateResponse(Run):
    pass


class RunResumeRequest(BaseModel):
    await_resume: AwaitResume
    mode: RunMode


class RunResumeResponse(Run):
    pass


class RunReadResponse(Run):
    pass


class RunCancelResponse(Run):
    pass


class RunEventsListResponse(BaseModel):
    events: list[Event]


class SessionReadResponse(Session):
    pass



import asyncio

import pytest
from acp_sdk.client import Client
from acp_sdk.models import (
    Message,
    MessagePart,
)
from acp_sdk.server import Server

agent = "history_echo"
input = [Message(parts=[MessagePart(content="Hello!")])]
output = [message.model_copy(update={"role": f"agent/{agent}"}) for message in input]


@pytest.mark.asyncio
async def test_session(server: Server, client: Client) -> None:
    async with client.session() as session:
        run = await session.run_sync(agent=agent, input=input)
        assert run.output == output
        run = await session.run_sync(agent=agent, input=input)
        assert run.output == output * 3


@pytest.mark.asyncio
async def test_session_refresh(server: Server, client: Client) -> None:
    async with client.session() as session:
        await session.run_async(agent=agent, input=input)
        await asyncio.sleep(2)
        sess = await session.refresh_session()
        assert len(sess.history) == len(input) * 2


@pytest.mark.asyncio
async def test_distributed_session(multi_server: tuple[Server, Server]) -> None:
    one, two = multi_server
    one_url = f"http://localhost:{one.server.config.port}"
    two_url = f"http://localhost:{two.server.config.port}"
    async with Client() as client, client.session() as session:
        run = await session.run_sync(input, agent=agent, base_url=one_url)
        assert run.output == output
        run = await session.run_sync(input, agent=agent, base_url=two_url)
        assert run.output == output * 3
        run = await session.run_sync(input, agent=agent, base_url=one_url)
        assert run.output == output * 7



import re
import uuid
from typing import Callable

import cachetools

from acp_sdk.models import ResourceUrl
from acp_sdk.models.types import ResourceId
from acp_sdk.shared.resources import ResourceLoader, ResourceStore


class ServerResourceLoader(ResourceLoader):
    def __init__(
        self,
        *,
        loader: ResourceLoader,
        store: ResourceStore,
        create_resource_url: Callable[[ResourceId], ResourceUrl] | None,
    ) -> None:
        self._loader = loader
        self._store = store

        placeholder_id = uuid.uuid4()
        self._url_pattern = (
            re.escape(str(create_resource_url(placeholder_id))).replace(re.escape(str(placeholder_id)), r"([^/?&#]+)")
            if create_resource_url
            else None
        )

    @cachetools.func.lfu_cache
    async def load(self, url: ResourceUrl) -> bytes:
        if self._url_pattern:
            match = re.match(self._url_pattern, str(url))
            if match:
                id = ResourceId(match.group(1))
                result = await self._store.load(id)
                return (await result.bytes_async()).to_bytes()
        return await self._loader.load(url)



from acp_sdk.models.errors import *  # noqa: F403
from acp_sdk.models.models import *  # noqa: F403
from acp_sdk.models.schemas import *  # noqa: F403
from acp_sdk.models.types import *  # noqa: F403



import asyncio
import os
import re
from collections.abc import AsyncGenerator, Awaitable
from contextlib import asynccontextmanager
from typing import Any, Callable

import requests
import uvicorn
import uvicorn.config
from fastapi import FastAPI

from acp_sdk.models import Metadata
from acp_sdk.server.agent import AgentManifest
from acp_sdk.server.agent import agent as agent_decorator
from acp_sdk.server.app import create_app
from acp_sdk.server.logging import configure_logger as configure_logger_func
from acp_sdk.server.logging import logger
from acp_sdk.server.store import Store
from acp_sdk.server.telemetry import configure_telemetry as configure_telemetry_func
from acp_sdk.server.utils import async_request_with_retry
from acp_sdk.shared.resources import ResourceLoader, ResourceStore


class Server:
    def __init__(self) -> None:
        self.agents: list[AgentManifest] = []
        self.server: uvicorn.Server | None = None

    def agent(
        self,
        name: str | None = None,
        description: str | None = None,
        *,
        metadata: Metadata | None = None,
    ) -> Callable:
        """Decorator to register an agent."""

        def decorator(fn: Callable) -> Callable:
            agent = agent_decorator(name=name, description=description, metadata=metadata)(fn)
            self.register(agent)
            return fn

        return decorator

    def register(self, *agents: AgentManifest) -> None:
        self.agents.extend(agents)

    @asynccontextmanager
    async def lifespan(self, app: FastAPI) -> AsyncGenerator[None]:
        yield

    async def serve(
        self,
        *,
        configure_logger: bool = True,
        configure_telemetry: bool = False,
        self_registration: bool = True,
        store: Store | None = None,
        resource_store: ResourceStore | None = None,
        resource_loader: ResourceLoader | None = None,
        host: str = "127.0.0.1",
        port: int = 8000,
        uds: str | None = None,
        fd: int | None = None,
        loop: uvicorn.config.LoopSetupType = "auto",
        http: type[asyncio.Protocol] | uvicorn.config.HTTPProtocolType = "auto",
        ws: type[asyncio.Protocol] | uvicorn.config.WSProtocolType = "auto",
        ws_max_size: int = 16 * 1024 * 1024,
        ws_max_queue: int = 32,
        ws_ping_interval: float | None = 20.0,
        ws_ping_timeout: float | None = 20.0,
        ws_per_message_deflate: bool = True,
        lifespan: uvicorn.config.LifespanType = "auto",
        env_file: str | os.PathLike[str] | None = None,
        log_config: dict[str, Any]
        | str
        | uvicorn.config.RawConfigParser
        | uvicorn.config.IO[Any]
        | None = uvicorn.config.LOGGING_CONFIG,
        log_level: str | int | None = None,
        access_log: bool = True,
        use_colors: bool | None = None,
        interface: uvicorn.config.InterfaceType = "auto",
        reload: bool = False,
        reload_dirs: list[str] | str | None = None,
        reload_delay: float = 0.25,
        reload_includes: list[str] | str | None = None,
        reload_excludes: list[str] | str | None = None,
        workers: int | None = None,
        proxy_headers: bool = True,
        server_header: bool = True,
        date_header: bool = True,
        forwarded_allow_ips: list[str] | str | None = None,
        root_path: str = "",
        limit_concurrency: int | None = None,
        limit_max_requests: int | None = None,
        backlog: int = 2048,
        timeout_keep_alive: int = 5,
        timeout_notify: int = 30,
        timeout_graceful_shutdown: int | None = None,
        callback_notify: Callable[..., Awaitable[None]] | None = None,
        ssl_keyfile: str | os.PathLike[str] | None = None,
        ssl_certfile: str | os.PathLike[str] | None = None,
        ssl_keyfile_password: str | None = None,
        ssl_version: int = uvicorn.config.SSL_PROTOCOL_VERSION,
        ssl_cert_reqs: int = uvicorn.config.ssl.CERT_NONE,
        ssl_ca_certs: str | None = None,
        ssl_ciphers: str = "TLSv1",
        headers: list[tuple[str, str]] | None = None,
        factory: bool = False,
        h11_max_incomplete_event_size: int | None = None,
    ) -> None:
        if self.server:
            raise RuntimeError("The server is already running")

        if headers is None:
            headers = [("server", "acp")]
        elif not any(k.lower() == "server" for k, _ in headers):
            headers.append(("server", "acp"))

        import uvicorn

        app = create_app(
            *self.agents,
            lifespan=self.lifespan,
            store=store,
            resource_loader=resource_loader,
            resource_store=resource_store,
        )

        if configure_logger:
            configure_logger_func()
        if configure_telemetry:
            configure_telemetry_func(app)

        config = uvicorn.Config(
            app,
            host,
            port,
            uds,
            fd,
            loop,
            http,
            ws,
            ws_max_size,
            ws_max_queue,
            ws_ping_interval,
            ws_ping_timeout,
            ws_per_message_deflate,
            lifespan,
            env_file,
            log_config,
            log_level,
            access_log,
            use_colors,
            interface,
            reload,
            reload_dirs,
            reload_delay,
            reload_includes,
            reload_excludes,
            workers,
            proxy_headers,
            server_header,
            date_header,
            forwarded_allow_ips,
            root_path,
            limit_concurrency,
            limit_max_requests,
            backlog,
            timeout_keep_alive,
            timeout_notify,
            timeout_graceful_shutdown,
            callback_notify,
            ssl_keyfile,
            ssl_certfile,
            ssl_keyfile_password,
            ssl_version,
            ssl_cert_reqs,
            ssl_ca_certs,
            ssl_ciphers,
            headers,
            factory,
            h11_max_incomplete_event_size,
        )
        self.server = uvicorn.Server(config)
        await self._serve(self_registration=self_registration)

    def run(
        self,
        *,
        configure_logger: bool = True,
        configure_telemetry: bool = False,
        self_registration: bool = True,
        store: Store | None = None,
        resource_store: ResourceStore | None = None,
        resource_loader: ResourceLoader | None = None,
        host: str = "127.0.0.1",
        port: int = 8000,
        uds: str | None = None,
        fd: int | None = None,
        loop: uvicorn.config.LoopSetupType = "auto",
        http: type[asyncio.Protocol] | uvicorn.config.HTTPProtocolType = "auto",
        ws: type[asyncio.Protocol] | uvicorn.config.WSProtocolType = "auto",
        ws_max_size: int = 16 * 1024 * 1024,
        ws_max_queue: int = 32,
        ws_ping_interval: float | None = 20.0,
        ws_ping_timeout: float | None = 20.0,
        ws_per_message_deflate: bool = True,
        lifespan: uvicorn.config.LifespanType = "auto",
        env_file: str | os.PathLike[str] | None = None,
        log_config: dict[str, Any]
        | str
        | uvicorn.config.RawConfigParser
        | uvicorn.config.IO[Any]
        | None = uvicorn.config.LOGGING_CONFIG,
        log_level: str | int | None = None,
        access_log: bool = True,
        use_colors: bool | None = None,
        interface: uvicorn.config.InterfaceType = "auto",
        reload: bool = False,
        reload_dirs: list[str] | str | None = None,
        reload_delay: float = 0.25,
        reload_includes: list[str] | str | None = None,
        reload_excludes: list[str] | str | None = None,
        workers: int | None = None,
        proxy_headers: bool = True,
        server_header: bool = True,
        date_header: bool = True,
        forwarded_allow_ips: list[str] | str | None = None,
        root_path: str = "",
        limit_concurrency: int | None = None,
        limit_max_requests: int | None = None,
        backlog: int = 2048,
        timeout_keep_alive: int = 5,
        timeout_notify: int = 30,
        timeout_graceful_shutdown: int | None = None,
        callback_notify: Callable[..., Awaitable[None]] | None = None,
        ssl_keyfile: str | os.PathLike[str] | None = None,
        ssl_certfile: str | os.PathLike[str] | None = None,
        ssl_keyfile_password: str | None = None,
        ssl_version: int = uvicorn.config.SSL_PROTOCOL_VERSION,
        ssl_cert_reqs: int = uvicorn.config.ssl.CERT_NONE,
        ssl_ca_certs: str | None = None,
        ssl_ciphers: str = "TLSv1",
        headers: list[tuple[str, str]] | None = None,
        factory: bool = False,
        h11_max_incomplete_event_size: int | None = None,
    ) -> None:
        asyncio.run(
            self.serve(
                configure_logger=configure_logger,
                configure_telemetry=configure_telemetry,
                self_registration=self_registration,
                store=store,
                resource_store=resource_store,
                resource_loader=resource_loader,
                host=host,
                port=port,
                uds=uds,
                fd=fd,
                loop=loop,
                http=http,
                ws=ws,
                ws_max_size=ws_max_size,
                ws_max_queue=ws_max_queue,
                ws_ping_interval=ws_ping_interval,
                ws_ping_timeout=ws_ping_timeout,
                ws_per_message_deflate=ws_per_message_deflate,
                lifespan=lifespan,
                env_file=env_file,
                log_config=log_config,
                log_level=log_level,
                access_log=access_log,
                use_colors=use_colors,
                interface=interface,
                reload=reload,
                reload_dirs=reload_dirs,
                reload_delay=reload_delay,
                reload_includes=reload_includes,
                reload_excludes=reload_excludes,
                workers=workers,
                proxy_headers=proxy_headers,
                server_header=server_header,
                date_header=date_header,
                forwarded_allow_ips=forwarded_allow_ips,
                root_path=root_path,
                limit_concurrency=limit_concurrency,
                limit_max_requests=limit_max_requests,
                backlog=backlog,
                timeout_keep_alive=timeout_keep_alive,
                timeout_notify=timeout_notify,
                timeout_graceful_shutdown=timeout_graceful_shutdown,
                callback_notify=callback_notify,
                ssl_keyfile=ssl_keyfile,
                ssl_certfile=ssl_certfile,
                ssl_keyfile_password=ssl_keyfile_password,
                ssl_version=ssl_version,
                ssl_cert_reqs=ssl_cert_reqs,
                ssl_ca_certs=ssl_ca_certs,
                ssl_ciphers=ssl_ciphers,
                headers=headers,
                factory=factory,
                h11_max_incomplete_event_size=h11_max_incomplete_event_size,
            )
        )

    async def _serve(self, self_registration: bool = True) -> None:
        registration_task = asyncio.create_task(self._register_agent()) if self_registration else None
        await self.server.serve()
        if registration_task:
            registration_task.cancel()

    @property
    def should_exit(self) -> bool:
        return self.server.should_exit if self.server else False

    @should_exit.setter
    def should_exit(self, value: bool) -> None:
        self.server.should_exit = value

    async def _register_agent(self) -> None:
        """If not in PRODUCTION mode, register agent to the beeai platform and provide missing env variables"""
        if os.getenv("PRODUCTION_MODE", "").lower() in ["true", "1"]:
            logger.debug("Agent is not automatically registered in the production mode.")
            return

        url = os.getenv("PLATFORM_URL", "http://127.0.0.1:8333")
        host = re.sub(r"localhost|127\.0\.0\.1", "host.docker.internal", self.server.config.host)
        request_data = {
            "location": f"http://{host}:{self.server.config.port}",
        }
        await async_request_with_retry(lambda client, data=request_data: client.get(f"{url}/api/v1/providers"))
        try:
            await async_request_with_retry(
                lambda client, data=request_data: client.post(
                    f"{url}/api/v1/providers", json=data, params={"auto_remove": True}
                )
            )
            logger.info("Agent registered to the beeai server.")

            # check missing env keyes
            envs_request = await async_request_with_retry(lambda client: client.get(f"{url}/api/v1/variables"))
            envs = envs_request.get("env")
            os.environ["LLM_MODEL"] = "dummy"
            os.environ["LLM_API_KEY"] = "dummy"
            os.environ["LLM_API_BASE"] = f"{url.rstrip('/')}/api/v1/llm"

            for agent in self.agents:
                # register all available envs
                missing_keyes = []
                for env in agent.metadata.model_dump().get("env", []):
                    # Those envs are set to use LLM gateway from platform server
                    if env["name"] in {"LLM_MODEL", "LLM_API_KEY", "LLM_API_BASE"}:
                        continue
                    server_env = envs.get(env.get("name"))
                    if server_env:
                        logger.debug(f"Env variable {env['name']} = '{server_env}' added dynamically")
                        os.environ[env["name"]] = server_env
                    elif env.get("required"):
                        missing_keyes.append(env)
                if len(missing_keyes):
                    logger.error(f"Can not run agent, missing required env variables: {missing_keyes}")
                    raise Exception("Missing env variables")

        except requests.exceptions.ConnectionError as e:
            logger.warning(f"Can not reach server, check if running on {url} : {e}")
        except (requests.exceptions.HTTPError, Exception) as e:
            try:
                error_message = e.response.json().get("detail")
                if error_message:
                    logger.warning(f"Agent can not be registered to beeai server: {error_message}")
                else:
                    logger.warning(f"Agent can not be registered to beeai server: {e}")
            except Exception:
                logger.warning(f"Agent can not be registered to beeai server: {e}")



from enum import Enum

from pydantic import BaseModel, ConfigDict


class PlatformUIType(str, Enum):
    CHAT = "chat"
    HANDSOFF = "hands-off"


class AgentToolInfo(BaseModel):
    name: str
    description: str | None = None
    model_config = ConfigDict(extra="allow")


class PlatformUIAnnotation(BaseModel):
    ui_type: PlatformUIType
    user_greeting: str | None = None
    display_name: str | None = None
    tools: list[AgentToolInfo] = []
    model_config = ConfigDict(extra="allow")



from concurrent.futures import ThreadPoolExecutor

import janus
from fastapi import Request

from acp_sdk.models import Session
from acp_sdk.server.types import RunYield, RunYieldResume
from acp_sdk.shared import ResourceLoader, ResourceStore


class Context:
    def __init__(
        self,
        *,
        session: Session,
        store: ResourceStore,
        loader: ResourceLoader,
        executor: ThreadPoolExecutor,
        request: Request,
        yield_queue: janus.Queue[RunYield],
        yield_resume_queue: janus.Queue[RunYieldResume],
    ) -> None:
        self.session = session
        self.storage = store
        self.loader = loader
        self.executor = executor
        self.request = request
        self._yield_queue = yield_queue
        self._yield_resume_queue = yield_resume_queue

    def yield_sync(self, value: RunYield) -> RunYieldResume:
        self._yield_queue.sync_q.put(value)
        return self._yield_resume_queue.sync_q.get()

    async def yield_async(self, value: RunYield) -> RunYieldResume:
        await self._yield_queue.async_q.put(value)
        return await self._yield_resume_queue.async_q.get()

    def shutdown(self) -> None:
        self._yield_queue.shutdown()
        self._yield_resume_queue.shutdown()



import asyncio

import pytest
from acp_sdk.models.errors import ACPError, Error, ErrorCode
from acp_sdk.models.models import Message, MessagePart, Run, RunStatus

timestamp = "2021-09-09T22:02:47.89Z"


@pytest.mark.parametrize(
    "first,second,result",
    [
        (
            Message(
                parts=[MessagePart(content_type="text/plain", content="Foo")],
                created_at=timestamp,
                completed_at=timestamp,
            ),
            Message(
                parts=[MessagePart(content_type="text/plain", content="Bar")],
                created_at=timestamp,
                completed_at=timestamp,
            ),
            Message(
                parts=[
                    MessagePart(content_type="text/plain", content="Foo"),
                    MessagePart(content_type="text/plain", content="Bar"),
                ],
                created_at=timestamp,
                completed_at=timestamp,
            ),
        ),
        (
            Message(
                parts=[MessagePart(content_type="text/plain", content="Foo")],
                created_at=None,
                completed_at=timestamp,
            ),
            Message(
                parts=[MessagePart(content_type="text/plain", content="Bar")],
                created_at=timestamp,
                completed_at=None,
            ),
            Message(
                parts=[
                    MessagePart(content_type="text/plain", content="Foo"),
                    MessagePart(content_type="text/plain", content="Bar"),
                ],
                created_at=None,
                completed_at=None,
            ),
        ),
    ],
)
def test_message_add(first: Message, second: Message, result: Message) -> None:
    assert first + second == result


@pytest.mark.parametrize(
    "uncompressed,compressed",
    [
        (
            Message(
                parts=[
                    MessagePart(content_type="text/plain", content="Foo"),
                    MessagePart(content_type="text/plain", content="Bar"),
                ],
                created_at=timestamp,
                completed_at=timestamp,
            ),
            Message(
                parts=[MessagePart(content_type="text/plain", content="FooBar")],
                created_at=timestamp,
                completed_at=timestamp,
            ),
        ),
        (
            Message(
                parts=[
                    MessagePart(content_type="text/plain", content="Foo"),
                    MessagePart(content_type="text/html", content="<head>"),
                    MessagePart(content_type="text/plain", content="Foo"),
                    MessagePart(content_type="text/plain", content="Bar"),
                ],
                created_at=timestamp,
                completed_at=timestamp,
            ),
            Message(
                parts=[
                    MessagePart(content_type="text/plain", content="Foo"),
                    MessagePart(content_type="text/html", content="<head>"),
                    MessagePart(content_type="text/plain", content="FooBar"),
                ],
                created_at=timestamp,
                completed_at=timestamp,
            ),
        ),
    ],
)
def test_message_compress(uncompressed: Message, compressed: Message) -> None:
    assert uncompressed.compress() == compressed


@pytest.mark.parametrize(
    "run,error",
    [
        (
            Run(agent_name="foo", status=RunStatus.CANCELLED),
            asyncio.CancelledError,
        ),
        (
            Run(
                agent_name="foo",
                status=RunStatus.FAILED,
                error=Error(code=ErrorCode.SERVER_ERROR, message="Unspecified"),
            ),
            ACPError,
        ),
        (
            Run(agent_name="foo"),
            None,
        ),
        (
            Run(agent_name="foo", status=RunStatus.IN_PROGRESS),
            None,
        ),
        (
            Run(agent_name="foo", status=RunStatus.COMPLETED),
            None,
        ),
    ],
)
def test_run_raise_on_status_raise(run: Run, error: type[Exception] | None) -> None:
    if error:
        with pytest.raises(error):
            run.raise_for_status()
    else:
        run.raise_for_status()
